/* modified lock.c by sivann */
/* accept shadow passwords/other changes  */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <sgtty.h>
#include <shadow.h>


void            quit(), bye(), sig_hdl();
struct timeval  timeout;
struct timeval  zerotime;
struct sgttyb   tty, ntty;
long            nexttime;	/* keep the timeout time */
long 		count=0;
int 		usemine;
int		sectimeout=-1;

int main(int argc, char **argv)
{
  extern char    *optarg;
  extern int      errno, optind;
  struct spwd  *spw;
  struct timeval  timval;
  struct itimerval ntimer, otimer;
  struct tm      *timp;
  int             ch ;
  char           *ap, *mypw, *ttynam;
  char            hostname[128], s[BUFSIZ], s1[BUFSIZ];
  char           *crypt(), *ttyname();

  mypw = NULL;
  usemine = 1;

  while ((ch = getopt(argc, argv, "it:")) != EOF)
    switch ((char) ch) {
    case 't':
      if ((sectimeout = atoi(optarg)) <= 0) {
	 fprintf(stderr, "%s: illegal timeout value.\n",argv[0]);
	exit(1);
      }
      break;
    case 'i':
      usemine = 0;
      break;
    case '?':
    default:
       fprintf(stderr,
		     "usage: %s [-i] [-t timeout]\n"
		     "\t\t-i:\task for password\n"
		     "\t\t-t:\ttimeout in seconds\n",argv[0]);
      exit(1);
    }

  if (usemine) {
    if (!(spw = getspnam(getenv("USER")))) {
       fprintf(stderr, "%s: invalid $USER\n",argv[0]);
      exit(1);
    }
    mypw = strdup(spw->sp_pwdp);
  }


  setuid(getuid());		/* discard privs */
  
  /* get information for header */
  if (ioctl(0, TIOCGETP, &tty))	{
    perror("ioctl");
    exit(errno);
  }

  if (!(ttynam = ttyname(0))) {
     printf("%s: not a tty\n",argv[0]);
    exit(1);
  }

  gethostname(hostname, sizeof(hostname));

  gettimeofday(&timval, (struct timezone *) NULL);

  timeout.tv_sec = sectimeout * 60;
  nexttime = timval.tv_sec + (sectimeout * 60);
  timp = localtime(&timval.tv_sec);
  ap = asctime(timp);

  signal(SIGINT, quit);
  signal(SIGQUIT, quit);

  ntty = tty;
  ntty.sg_flags &= ~ECHO;
  ioctl(0, TIOCSETP, &ntty);

  if (!mypw) {
    /* get key */
    printf("Key: ");

    if (!fgets(s, sizeof(s), stdin) || *s == '\n')
      quit();

     printf("\nAgain: ");

    /* Don't need EOF test here, if we get EOF, then s1 != s and the right
     * things will happen. */
     fgets(s1, sizeof(s1), stdin);
     putchar('\n');

    if (strcmp(s1, s)) {
       printf("\07%s: passwords didn't match.\n",argv[0]);
      ioctl(0, TIOCSETP, &tty);
      exit(1);
    }
    s[0] = NULL;
    mypw = s1;
  }

  /* set signal handlers */
  signal(SIGINT, sig_hdl);
  signal(SIGQUIT, sig_hdl);
  signal(SIGTSTP, sig_hdl);
  signal(SIGALRM, bye);

  ntimer.it_interval = zerotime;
  ntimer.it_value = timeout;
  if (sectimeout>0)
    setitimer(ITIMER_REAL, &ntimer, &otimer);

  /* header info */
  if (sectimeout>0)
   printf("lock: %s on %s. timeout in %d minutes\ntime now is %.20s%s",
		ttynam, hostname, sectimeout, ap, ap + 19);

  for (;;) {
    if (usemine)
      printf("Password: ");
    else
      printf("Key: ");

    if (!fgets(s, sizeof(s), stdin)) {
      clearerr(stdin);
      sig_hdl();
      continue;
    }
    if (usemine) {
      s[strlen(s) - 1] = '\0';
      if (!strcmp(mypw, crypt(s, mypw)))
	break;
    }
    else if (!strcmp(s, s1)) {
      break;
    }
    printf(" Incorrect!\07\n");
    count++;

    if (ioctl(0, TIOCGETP, &ntty))
      exit(1);
  }

  quit();
  return 5;
}

void
sig_hdl()
{
  struct timeval  timval;

   signal(SIGINT, sig_hdl);
   signal(SIGQUIT, sig_hdl);
   signal(SIGTSTP, sig_hdl);

  if (!usemine) 
    printf("type in the unlock key. ");
  else 
    printf("type in your password. ");

  if (sectimeout>0 && !gettimeofday(&timval, (struct timezone *) NULL)) 
      printf("timeout in %ld:%ld minutes\n",
	  (nexttime - timval.tv_sec) / 60, 
	  (nexttime - timval.tv_sec) % 60);
  else 
    printf("\n");

}

void
quit()
{
  putchar('\n');
  ioctl(0, TIOCSETP, &tty);
  printf("Succeded with %ld tries!\n",count);
  exit(count);
}

void
bye()
{
  ioctl(0, TIOCSETP, &tty);
  printf(" timeout\n");
  exit(1);
}
